
options = (
    ('trusted-vcs-repositories',
     {'type' : 'yn',
      'default': True,
      'help': 'Are repositories trusted, in which case checkin messages are '
      'analyzed to trigger some actions such as changing ticket\'states.',
      'level': 2, 'group': 'vcsfile',
      }),
    )

